# -*- coding: utf-8 -*-
'''
Copyright 2012 Rodrigo Pinheiro Matias <rodrigopmatias@gmail.com>

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

   http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
'''
from hwm.plugin.base import BaseColectorPlugin, BaseGraphicPlugin
from hwm.humanize import bytes

import re
import os
import subprocess

class Memory(BaseColectorPlugin, BaseGraphicPlugin):
    
    @classmethod
    def description(class_):
        dct = BaseColectorPlugin.description()
        dct.update({
            'name': 'Memory',
            'slug': 'memory'
        })

        return dct

    def _prepare(self, label, total, free, active, inactive):
        dbc = self.get_connector()
        cursor = dbc.cursor()

        try:
            cursor.execute('''
CREATE TABLE IF NOT EXISTS memory(
    ID INTEGER PRIMARY KEY AUTOINCREMENT,
    LABEL VARCHAR(100) NOT NULL,
    COLECT_MOMENT TIMESTAMP DEFAULT CURRENT_TIMESTAMP,
    TOTAL INTEGER NOT NULL,
    FREE INTEGER NOT NULL,
    ACTIVE INTEGER NOT NULL,
    INACTIVE INTEGER NOT NULL
)''')
        except Exception, e:
            print e
        else:
            try:
                cursor.execute(
                    'INSERT INTO memory(LABEL,TOTAL,ACTIVE,INACTIVE,FREE) VALUES(:label, :total, :active, :inactive, :free)',
                    vars()
                )
            except Exception, e:
                print e
            else:
                dbc.commit()

        cursor.close()
        dbc.close()

    def do_colect(self):
        with open('/proc/meminfo', 'r') as fd:
            text = fd.read()
            text = text.replace('\n', ' ')
            rst = re.match(r'^.*MemTotal:\s+(?P<total>\d+).*MemFree:\s+(?P<free>\d+).*Active:\s+(?P<active>\d+).*Inactive:\s+(?P<inactive>\d+).*$', text)
            if rst is not None: 
                params = rst.groupdict()
                params.update(label = self._params.label)
                self._prepare(**params)

    def get_column(self):
        return ('total', 'free', 'active', 'inactive')

    def do_extract(self, label, size = 10):
        dbc = self.get_connector()
        sql = '''
SELECT
    total,
    free,
    active,
    inactive
FROM
    memory
WHERE
    label = :label
ORDER BY
    id
LIMIT :size'''

        count = 0
        for row in dbc.execute(sql, vars()):
            yield row
            count += 1

        for x in xrange(count, size):
            yield (0, 0, 0, 0)

        dbc.close()

    @property
    def filename(self):
        return '%s.png' % self._params.label

    @property
    def colors(self):
        return (
            '0000ff',
            '00ff00',
            'ff0000',
            'eeeeee'
        )

    @property
    def params(self):
        return {
            'label': self._params.label,
            'size': 60
        }

    def humanize(self, number):
        return bytes(number, 2, 1)