# -*- coding: utf-8 -*-
'''
Copyright 2012 Rodrigo Pinheiro Matias <rodrigopmatias@gmail.com>

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

   http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
'''

from hwm.plugin.base import BaseColectorPlugin, BaseGraphicPlugin, font
from hwm.humanize import bytes
from datetime import datetime
try:
    from PIL import ImageColor
except ImportError:
    import ImageColor

import subprocess
import re
import sys
import os

'''
Tempo de atualização dos dados.
'''
TIME_UPDATE = float(os.environ.get('TIME_UPDATE', (60 * 1)))

class Network(BaseColectorPlugin, BaseGraphicPlugin):

    def _legend(self, canvas):
        BaseGraphicPlugin._legend(self, canvas)
        self._legend_min(canvas)
        self._legend_avg(canvas)
        self._legend_max(canvas)
        self._legend_total(canvas)

    def _legend_total(self, canvas):
        data = BaseColectorPlugin.get_data_sequence(self, **self.params)
        base = {}
        
        for column in self.get_column():
            base.update({column: []})

        for row in data.get('collection', []):
            sample = dict(zip(self.get_column(), row))
            for column in base.keys():
                if sample.get(column, 0) > 0:
                    base.get(column, []).append(sample.get(column, 0))
                else:
                    base.get(column, []).append(0)

        legends = dict(zip(self.get_column(), self.colors))
        x, y = 520, 200
        for text, color in legends.items():
            total = sum(base.get(text))
            canvas.text(
                (x + 15, y), 
                'Total %s: %s' % (text, '%0.2f %s' % (self.humanize(total))),
                ** {
                    'fill': ImageColor.getcolor('#000', 'RGB'),
                    'font': font
                }
            )
            y += 15

    def _legend_max(self, canvas):
        data = BaseColectorPlugin.get_data_sequence(self, **self.params)
        base = dict(zip(self.get_column(), [0] * len(self.get_column())))

        for row in data.get('collection', []):
            sample = dict(zip(self.get_column(), row))
            for column in base.keys():
                if sample.get(column) > base.get(column):
                    base.update({column: sample.get(column)})


        legends = dict(zip(self.get_column(), self.colors))
        x, y = 370, 200
        for text, color in legends.items():
            canvas.text(
                (x + 15, y), 
                'Max %s: %s' % (text, '%0.2f %s' % (self.humanize(base.get(text, 0)))),
                ** {
                    'fill': ImageColor.getcolor('#000', 'RGB'),
                    'font': font
                }
            )

            y += 15

    def _legend_avg(self, canvas):
        data = BaseColectorPlugin.get_data_sequence(self, **self.params)
        base = {}
        
        for column in self.get_column():
            base.update({column: []})

        for row in data.get('collection', []):
            sample = dict(zip(self.get_column(), row))
            for column in base.keys():
                if sample.get(column, 0) > 0:
                    base.get(column, []).append(sample.get(column, 0))
                else:
                    base.get(column, []).append(0)

        legends = dict(zip(self.get_column(), self.colors))
        x, y = 220, 200
        for text, color in legends.items():
            avg = sum(base.get(text)) / float(len(base.get(text)))
            canvas.text(
                (x + 15, y), 
                'Avg %s: %s' % (text, '%0.2f %s' % (self.humanize(avg))),
                ** {
                    'fill': ImageColor.getcolor('#000', 'RGB'),
                    'font': font
                }
            )
            y += 15

    def _legend_min(self, canvas):
        data = BaseColectorPlugin.get_data_sequence(self, **self.params)
        base = dict(zip(self.get_column(), [0] * len(self.get_column())))

        for row in data.get('collection', []):
            sample = dict(zip(self.get_column(), row))
            for column in base.keys():
                if (sample.get(column) < base.get(column) or base.get(column) == 0) and sample.get(column) > 0:
                    base.update({column: sample.get(column)})

        legends = dict(zip(self.get_column(), self.colors))
        x, y = 80, 200
        for text, color in legends.items():
            canvas.text(
                (x + 15, y), 
                'Min %s: %s' % (text, '%0.2f %s' % (self.humanize(base.get(text, 0)))),
                ** {
                    'fill': ImageColor.getcolor('#000', 'RGB'),
                    'font': font
                }
            )

            y += 15

    def clear(self):
        dbc = self.get_connector()
        cursor = dbc.cursor()

        try:
            cursor.execute(
                'DELETE FROM net_trafic WHERE interface = :interface and label = :label', 
                self._params.__dict__
            )
        except Exception, e:
            print 'Ocorreu um erro executando a limpeza.'
            print e
        else:
            dbc.commit()

    def __init__(self, *args, **kargs):
        BaseColectorPlugin.__init__(self, *args, **kargs)
        self._period = TIME_UPDATE

    def get_column(self):
        return tuple(['in', 'out'])

    def humanize(self, number):
        return bytes(number)
    
    @classmethod
    def description(class_):
        dct = BaseColectorPlugin.description()
        dct.update({
            'name': 'Network',
            'slug': 'network',
            'options': [
                (
                    '--interface',
                    {
                        'help': 'Especify the Interface.',
                        'dest': 'interface'
                    }
                ),
                (
                    '--label',
                    {
                        'help': 'Especify the label of colect.',
                        'dest': 'label'
                    }
                )
            ]
        })

        return dct

    @property
    def params(self):
        return {
            'interface': self._params.interface,
            'label': self._params.label,
            'size': 60,
            'path': getattr(self._params, 'path', None)

        }

    @property
    def filename(self):
        filename = '%(label)s.png' % self.params
        if self.params.get('path', None) is not None:
            filename = os.path.join(path, filename)
        return filename

    @property
    def colors(self):
        return ('0c94bb', '050574')

    def _prepare(self, **kargs):
        dbc = self.get_connector()
        cursor = dbc.cursor()

        try:
            cursor.execute(
                '''
CREATE TABLE IF NOT EXISTS net_trafic(
    ID INTEGER PRIMARY KEY AUTOINCREMENT,
    INTERFACE VARCHAR(300) NOT NULL,
    RX INTEGER NOT NULL DEFAULT 0,
    TX INTEGER NOT NULL DEFAULT 0,
    LABEL VARCHAR(40) NOT NULL,
    COLECT_MOMENT TIMESTAMP DEFAULT CURRENT_TIMESTAMP
)
                '''
            )
        except Exception, e:
            print e
        else:
            try:
                cursor.execute(
                    '''
INSERT INTO net_trafic(INTERFACE, LABEL, RX, TX) VALUES(:interface, :label, :rx, :tx)
                    ''',
                    kargs
                )
            except Exception, e:
                print e
            else:
                dbc.commit()

        cursor.close()
        dbc.close()

    def do_extract(self, size = 10, interface = 'eth0', label = None, path = None):
        dbc = self.get_connector()
        sql = '''
SELECT 
    RX, 
    TX,
    COLECT_MOMENT 
FROM
    net_trafic
WHERE
    INTERFACE = :interface and LABEL = :label
ORDER BY
    ID DESC
LIMIT :size
        '''
        params = {
            'size': size + 1,
            'interface': interface,
            'label': label
        }

        otx, orx = 0, 0
        count = 0
        for rx, tx, moment in dbc.execute(sql, params):
            yield (otx - tx), (orx - rx), datetime.strptime(moment, '%Y-%m-%d %H:%M:%S')
            otx, orx = tx, rx
            count += 1

        while count < size:
            yield (0, 0, datetime.now())
            count += 1

        dbc.close()

    
    def do_colect(self):
        for line in subprocess.Popen(['/sbin/ifconfig', self._params.interface], stdout = subprocess.PIPE).stdout:
            rst = re.match('^.+RX bytes:(?P<rx>[0-9]+)\s.*TX bytes:(?P<tx>[0-9]+).+$', line)
            if rst is not None:
                params = rst.groupdict()
                params.update(interface = self._params.interface, label = self._params.label)
                self._prepare(**params)